<?php

namespace Slovocast\Controller\Channel;

use Odan\Session\SessionInterface;
use Psr\Log\LoggerInterface;
use Slovocast\Controller\Controller;
use Psr\Http\Message\ResponseInterface as Response;
use Slovocast\Domain\Repository\Channel\ChannelRepositoryInterface;
use Slovocast\Domain\Entity\Channel;
use Slovocast\Domain\Repository\User\UserRepositoryInterface;

class CreateChannelAction extends Controller
{
    const CREATE_TEMPLATE_PATH = 'channel/create.twig';

    public function __construct(
        protected SessionInterface $session,
        protected LoggerInterface $logger,
        protected ChannelRepositoryInterface $channelRepository,
        protected UserRepositoryInterface $userInterface
    ) { }

    public function handle(): Response
    {
        $userData = $this->session->get('user');
        $formData = $this->request->getParsedBody();

        if ($userData['id'] != $formData['user_id']) {
            $this->session->getFlash()->add("error", "Unable to create channel.");
            $this->logger->error("Session User ID and Form User ID do no match.");
            return $this->render(self::CREATE_TEMPLATE_PATH)->withStatus(400);
        }

        $formData['explicit'] = $formData['explicit'] === "yes" ? true : false;

        try {
            $user = $this->userInterface->get($userData['id']);
            $channel = Channel::fromArray($formData);

            $channel = $this->channelRepository->create($channel, $user);
            $this->session->getFlash()->add("success", "Created new channel.");

            return $this->redirect("/dashboard", 302);
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            $this->session->getFlash()->add("error", "Unable to create new channel.");

            return $this->render(self::CREATE_TEMPLATE_PATH)->withStatus(500);
        }
    }
}
