<?php

namespace Slovocast\Controller;

use Odan\Session\SessionInterface;
use Psr\Log\LoggerInterface;
use Slovocast\Domain\Repository\Channel\ChannelRepositoryInterface;
use Slovocast\Domain\Repository\User\UserRepositoryInterface;
use Psr\Http\Message\ResponseInterface as Response;
use Slovocast\Exception\EntityNotFoundException;

class DashboardPage extends Controller
{
    public function __construct(
        protected UserRepositoryInterface $userRepository,
        protected ChannelRepositoryInterface $channelRepository,
        protected SessionInterface $session,
        protected LoggerInterface $logger,
    ) { }

    public function handle(): Response
    {
        $userData = $this->session->get("user");
        $user = $this->userRepository->get($userData['id']);

        try {
            $channel = $this->channelRepository->getFromUser($user);
        } catch (EntityNotFoundException $e) {
            $this->logger->info($e->getMessage(), [
                'user' => $user->getEmail(),
            ]);
            $channel = null;
        }

        return $this->render('dashboard.twig', [ 'channel' => $channel ]);
    }
}
