<?php

namespace Slovocast\Middleware;

use Odan\Session\SessionInterface;
use Psr\Http\Message\ResponseFactoryInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Ramsey\Uuid\Uuid;

class FormKeyMiddleware implements MiddlewareInterface
{
    const FORM_KEY = "form_key";

    public function __construct(
        private SessionInterface $session,
        private ResponseFactoryInterface $responseFactory
    ) { }

    public function process(Request $request, RequestHandler $handler): Response
    {
        if (!$this->session->has(self::FORM_KEY)) {
            $uuid = Uuid::uuid4();
            $this->session->set(self::FORM_KEY, $uuid->toString());
        }

        // check if the reuquest is a POST
        // check for the form_key value
        // return a 403 if the form key does not match
        if ($request->getMethod() === "POST") {
            $parsedBody = $request->getParsedBody();
            $sessionFormKey = $this->session->get(self::FORM_KEY);

            if (isset($parsedBody[self::FORM_KEY])) {
                if ($parsedBody[self::FORM_KEY] === $sessionFormKey) {
                    $this->session->delete(self::FORM_KEY);
                    return $handler->handle($request);
                }
            }

            $response = $this->responseFactory->createResponse(403, "Invalid Form Key");
            return $response;
        }

        return $handler->handle($request);
    }
}
